% std_myerpplot() - Copied and modified version of std_erpplot used for
%                   std_IcErpBackProjection.
%
% Usage: 
% [STUDY, erpData, allTimes, pgroup, pcond, pinter] = 
%                          std_myerpplot(STUDY, ALLEEG, EEG, varargin)
% Inputs
% STUDY, ALLEEG, EEG: EEGLAB variables.
% GUI               : Choose channel to backproject. ERP and STATS Params
%                     take effect. 

% Authors: Makoto Miyakoshi, JSPS/SCCN,INC,UCSD; Arnaud Delorme, CERCO, August, 2006-
% History:
% 02/07/13 ver 1.1 by Makoto. Bug fixed (tmpSTUDY...)
% 10/31/12 ver 1.0 by Makoto. Created.

% Copyright (C) Makoto Miyakoshi, Arnaud Delorme
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function [STUDY, erpData, allTimes, pgroup, pcond, pinter] = std_myerpplot(STUDY, ALLEEG, EEG, varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% collect user input %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%

% create a letter string
tmpstring = STUDY.backProj.channel{1,1};
if length(STUDY.backProj.channel)>1
    for n = 2:length(STUDY.backProj.channel)
        tmpstring = [tmpstring '|' STUDY.backProj.channel{n,1}];
    end
end

% launch GUI to collect user input
[userInput tmpSTUDY] = inputgui('title', 'std_ICbackprojection()', 'userdata', STUDY, ...
   'geom',  ...
   {{2 3 [0 0] [1 1]} {2 3 [1 0] [1 1]}...
    {2 3 [0 2] [1 1]} {2 3 [1 2] [1 1]}},...
   'uilist',...
   {{'style' 'text' 'string' 'Channel [select one]'} {'style' 'popupmenu' 'string' tmpstring 'tag' 'phaseplot' 'value' 1}...
    {'style' 'pushbutton' 'enable' 'on' 'string' 'erp params'   'Callback' 'TMP = get(gcbf, ''userdata''); TMP = pop_erpparams(TMP);  set(gcbf, ''userdata'', TMP); clear TMP;' }...
    {'style' 'pushbutton' 'enable' 'on' 'string' 'stats params' 'Callback' 'TMP = get(gcbf, ''userdata''); TMP = pop_statparams(TMP); set(gcbf, ''userdata'', TMP); clear TMP;'}});

% if tmpSTUDY is not empty, updata STUDY with tmpSTUDY
if isempty(userInput)
    return
else
    STUDY = tmpSTUDY;
end

% convert user input
selectedChannel = userInput{1,1};

% create varargin
varargin{1,1} = 'clusters';
varargin{1,2} = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% here starts the original code %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 2
    help std_erpplot;
    return;
end;
erpData = []; alltimes = []; 
pgroup = []; pcond = []; pinter = [];

% find datatype and default options
% ---------------------------------
dtype = 'erp';
for ind = 1:2:length(varargin)
    if strcmpi(varargin{ind}, 'datatype')
        dtype = varargin{ind+1}; 
    end;
end;

% get parameters
% --------------
eval( [ 'tmp = pop_' dtype 'params(STUDY, varargin{:});' ...
        'params = tmp.etc.' dtype 'params; clear tmp;' ] );
statstruct.etc = STUDY.etc; 
statstruct = pop_statparams(statstruct, varargin{:});
stats = statstruct.etc.statistics;
    
% potentially missing fields
% --------------------------
fields     = { 'filter' 'subtractsubjectmean' 'timerange' 'freqrange' 'topotime' 'topofreq' 'averagechan'};
defaultval = { [] 'off' [] [] [] [] };
for ind=1:length(fields)
    if ~isfield(params, fields{ind}), 
        params = setfield(params, fields{ind}, defaultval{ind}); 
    end;
end;

% decode parameters
% -----------------
options = mystruct(varargin);
options = myrmfield( options, myfieldnames(params));
options = myrmfield( options, myfieldnames(stats));
options = myrmfield( options, { 'threshold' 'statistics' } ); % for backward compatibility
[opt ] = finputcheck( options, ...
                             { 'design'      'integer' []              STUDY.currentdesign;
                               'plotstderr'  'string'  []              'off';
                               'channels'    'cell'    []              {};
                               'clusters'    'integer' []              [];
                               'datatype'    'string'  { 'erp','spec' } 'erp';
                               'mode'        'string'  []              ''; % for backward compatibility (now used for statistics)
                               'comps'       { 'string','integer' } [] []; % for backward compatibility
                               'statmode'    'string'  { 'subjects','common','trials' } 'subjects'; % ignored
                               'plotmode'    'string' { 'normal','condensed' }  'normal';
                               'unitx'       'string' { 'ms','Hz' }    'ms';
                               'plotsubjects' 'string' { 'on','off' }  'off';
                               'subject'     'string'  []              '' }, 'std_erpplot');
if isstr(opt), error(opt); end;

if isstr(opt.comps), opt.comps = []; opt.plotsubjects = 'on'; end;
if ~isempty(params.topofreq) && strcmpi(opt.datatype, 'spec'),  params.topotime  = params.topofreq; end;
if ~isempty(params.freqrange), params.timerange = params.freqrange; end;
datatypestr = upper(opt.datatype);
if strcmpi(datatypestr, 'spec'), datatypestr = 'Spectrum'; end;

% =======================================================================
% below this line, all the code should be non-specific to ERP or spectrum
% =======================================================================

allconditions = STUDY.design(opt.design).variable(1).value;
allgroups     = STUDY.design(opt.design).variable(2).value;
paired = { STUDY.design(opt.design).variable(1).pairing ...
           STUDY.design(opt.design).variable(2).pairing };

% for backward compatibility
% --------------------------
if strcmpi(opt.mode, 'comps'), opt.plotsubjects = 'on'; end;
if strcmpi(stats.singletrials, 'off') && ((~isempty(opt.subject) || ~isempty(opt.comps)))
    if strcmpi(stats.condstats, 'on') || strcmpi(stats.groupstats, 'on')
        stats.groupstats = 'off';
        stats.condstats   = 'off'; 
        disp('No statistics for single subject/component'); 
    end;
end;

if ~isnan(params.topotime) & length(opt.channels) < 5
    warndlg2(strvcat('ERP parameters indicate that you wish to plot scalp maps', 'Select at least 5 channels to plot topography'));
    return;
end;

plotcurveopt = {};
if length(opt.clusters) > 1
    plotcurveopt = { 'figure' 'off' }; 
    params.plotconditions = 'together';
    params.plotgroups     = 'together';
    stats.condstats  = 'off'; 
    stats.groupstats = 'off';
end;

alpha    = fastif(strcmpi(stats.mode, 'eeglab'), stats.eeglab.alpha, stats.fieldtrip.alpha);
mcorrect = fastif(strcmpi(stats.mode, 'eeglab'), stats.eeglab.mcorrect, stats.fieldtrip.mcorrect);
method   = fastif(strcmpi(stats.mode, 'eeglab'), stats.eeglab.method, ['Fieldtrip ' stats.fieldtrip.method ]);
plotcurveopt = { plotcurveopt{:} ...
   'ylim',           params.ylim, ...
   'threshold',      alpha ...
   'unitx'           opt.unitx, ...
   'filter',         params.filter, ...
   'plotgroups',     params.plotgroups, ...
   'plotconditions', params.plotconditions };

% feed data
erpData = STUDY.backProj.backProjectedData;

% original data size and trancated size
allTimes   = STUDY.backProj.times;
timeLimits = STUDY.etc.erpparams.timerange;

% translate plotWindow and statsWindow to timepoint
plotWindow = find(allTimes >= timeLimits(1) & allTimes <= timeLimits(2));

% apply window to the data
if     length(size(erpData{1,1}))==3 % multi channel selected
    for group = 1:size(erpData, 2)
        for condition = 1:size(erpData, 1)
            erpData{condition, group} = squeeze(erpData{condition, group}(selectedChannel,plotWindow,:));
        end
    end
    
elseif length(size(erpData{1,1}))==2 % channel is singleton
    for group = 1:size(erpData, 2)
        for condition = 1:size(erpData, 1)
            erpData{condition, group} = squeeze(erpData{condition, group}(plotWindow,:));
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% perform statistics %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%
stats.paired = paired;
[pcond pgroup pinter] = std_stat(erpData, stats);

[alltitles alllegends] = std_figtitle('threshold', alpha,...
            'plotsubjects', opt.plotsubjects, 'mcorrect', mcorrect,...
            'condstat', stats.condstats, 'cond2stat', stats.groupstats, ...
            'statistics', method, 'condnames', allconditions,...
            'cond2names', allgroups, ...
            'subject', opt.subject, 'valsunit', opt.unitx,...
            'vals', params.topotime, 'datatype', '',...
            'cond2group', params.plotgroups,...
            'condgroup', params.plotconditions,...
            'clustname', STUDY.backProj.channel{selectedChannel,1},...
            'compnames', ['from Cls ' num2str(STUDY.backProj.cluster)]);

%%%%%%%%%%%%%%%%%
%%% plot erps %%%
%%%%%%%%%%%%%%%%%
std_plotcurve(allTimes(plotWindow), erpData, 'condnames', allconditions,...
             'legend', alllegends, 'groupnames', allgroups,...
             'plotstderr', opt.plotstderr, 'titles', alltitles,...
             'groupstats', pgroup, 'condstats', pcond,...
             'interstats', pinter, 'plotsubjects', opt.plotsubjects,...
             plotcurveopt{:});

%%%%%%%%%%%%%%%%%%
%%% plot title %%%
%%%%%%%%%%%%%%%%%%        
% create a letter string for title
tmpstring = [num2str(size(STUDY.backProj.backProjectedData{1,1},3)) ' ' STUDY.group{1,1}];
if length(STUDY.group)>1
    for n = 2:length(STUDY.group)
        tmpstring = [tmpstring ', ' num2str(size(STUDY.backProj.backProjectedData{1,n},3)) ' ' STUDY.group{1,n}];
    end
end         
      
annotation(gcf,'textbox', [0 1 1 0],...
    'String', tmpstring, 'HorizontalAlignment','center', 'FontSize',12,...
    'FitBoxToText','off', 'LineStyle','none');
         
set(gcf,'name', 'std_MyErpPlot()', 'numbertitle','off');
axcopy(gca);



% remove fields and ignore fields who are absent
% ----------------------------------------------
function s = myrmfield(s, f);

for index = 1:length(f)
    if isfield(s, f{index})
        s = rmfield(s, f{index});
    end;
end;



% convert to structure (but take into account cells)
% --------------------------------------------------
function s = mystruct(v);

for index=1:length(v)
    if iscell(v{index})
        v{index} = { v{index} };
    end;
end;
try
    s = struct(v{:});
catch, error('Parameter error'); end;



% convert to structure (but take into account cells)
% --------------------------------------------------
function s = myfieldnames(v);

s = fieldnames(v);
if isfield(v, 'eeglab')
    s2 = fieldnames(v.eeglab);
    s = { s{:} s2{:} };
end;
if isfield(v, 'fieldtrip')
    s3 = fieldnames(v.fieldtrip);
    for index=1:length(s3)
        s3{index} = [ 'fieldtrip' s3{index} ];
    end;
    s = { s{:} s3{:} };
end;
